import { AppPage } from "./app.po";
import { ListPage } from "./list.po";
import { CalculatorPage } from "./calculator.po";
import { Navbar } from "./navbar.po";
import { browser } from "protractor";
import { SideNav } from "./sidenav.po";

/**
 * Clone calculators
 */
describe("Solveur - ", () => {
  let startPage: AppPage;
  let listPage: ListPage;
  let calcPage: CalculatorPage;
  let navbar: Navbar;
  let sidenav: SideNav;

  beforeEach(() => {
    startPage = new AppPage();
    listPage = new ListPage();
    calcPage = new CalculatorPage();
    navbar = new Navbar();
    sidenav = new SideNav();
  });

  it("load > calculate", async () => {
    await startPage.navigateTo();

    await navbar.clickMenuButton();
    await browser.sleep(200);

    await sidenav.clickLoadSessionButton();
    await browser.sleep(200);

    await sidenav.loadSessionFile("./session/session-solveur-chutes.json");
    await browser.sleep(200);

    expect(await navbar.getAllCalculatorTabs().count()).toBe(4);
    await navbar.clickCalculatorTab(3); // n°3 should be the latest

    // check input values
    expect(await calcPage.getInputById("Xinit").getAttribute("value")).toBe("0.5");
    expect(await calcPage.getInputById("Ytarget").getAttribute("value")).toBe("252");
    // check Nub to calculate
    const ntc = calcPage.getSelectById("select_target_nub");
    const ntcV = await calcPage.getSelectValueText(ntc);
    expect(ntcV).toContain("PAB : puissance / Puissance dissipée (PV)");
    // check searched Parameter
    const sp = calcPage.getSelectById("select_searched_param");
    const spV = await calcPage.getSelectValueText(sp);
    expect(spV).toContain("Z2 - Cote aval (PAB : chute)");

    // check that "compute" button is active
    const calcButton = calcPage.getCalculateButton();
    const disabledState = await calcButton.getAttribute("disabled");
    expect(disabledState).not.toBe("true");
    // click "compute" button
    await calcButton.click();
    // check that result is not empty
    const hasResults = await calcPage.hasResults();
    expect(hasResults).toBe(true);
  });

  it("create > feed > calculate > clone > calculate clone", async () => {
    await startPage.navigateTo();

    // 1. create empty Solveur
    await listPage.clickMenuEntryForCalcType(22); // Solveur
    await browser.sleep(500);

    // 2. create PAB:Chute, PAB:Nombre and PAB:Puissance linked to one another
    await navbar.clickNewCalculatorButton();
    await listPage.clickMenuEntryForCalcType(12); // PAB:Chute
    await browser.sleep(500);
    await navbar.clickNewCalculatorButton();

    await listPage.clickMenuEntryForCalcType(13); // PAB:Nombre
    await browser.sleep(500);
    // link DHT to PAB:Chute.DH
    const dht = calcPage.getInputById("DHT");
    await calcPage.setParamMode(dht, "link");
    // Calculate DH
    const dh_nombre = calcPage.getInputById("DH");
    await calcPage.setParamMode(dh_nombre, "cal");

    await navbar.clickNewCalculatorButton();
    await listPage.clickMenuEntryForCalcType(6); // PAB:Puissance
    await browser.sleep(500);
    // link DH to PAB:Nombre.DH
    const dh_puiss = calcPage.getInputById("DH");
    await calcPage.setParamMode(dh_puiss, "link");

    // Go back to Solveur
    await navbar.clickCalculatorTab(0);

    await calcPage.changeSelectValue(calcPage.getSelectById("select_target_nub"), 1); // "Puissance / PV"
    await browser.sleep(500);
    await calcPage.changeSelectValue(calcPage.getSelectById("select_searched_param"), 2); // "Chute / Z2"
    await browser.sleep(500);
    await calcPage.getInputById("Ytarget").sendKeys("318");

    // check that "compute" button is active
    const calcButton = calcPage.getCalculateButton();
    const disabledState = await calcButton.getAttribute("disabled");
    expect(disabledState).not.toBe("true");
    // click "compute" button
    await calcButton.click();
    // check that result is not empty
    const hasResults = await calcPage.hasResults();
    expect(hasResults).toBe(true);

    // otherwise clickCloneCalcButton() fails with "Element is not clickable at point"
    await browser.executeScript("window.scrollTo(0, 0);");
    await calcPage.clickCloneCalcButton();
    await browser.sleep(500);

    // 4. check existence of the cloned module
    expect(await navbar.getAllCalculatorTabs().count()).toBe(5);
    await navbar.clickCalculatorTab(4); // n°4 should be the latest

    // check that result is empty
    const hasResultsClone1 = await calcPage.hasResults();
    expect(hasResultsClone1).toBe(false);

    // check that "compute" button is active
    const calcButtonClone = calcPage.getCalculateButton();
    const disabledStateClone = await calcButtonClone.getAttribute("disabled");
    expect(disabledStateClone).not.toBe("true");
    // click "compute" button
    await calcButtonClone.click();
    // check that result is not empty
    const hasResultsClone2 = await calcPage.hasResults();
    expect(hasResultsClone2).toBe(true);
  });
});
