import { IObservable } from "jalhyd";

import { FormulaireBase } from "./form-base";
import { FieldSet } from "../../fieldset";
import { FormResultFixedVar } from "../form-result-fixedvar";

/**
 * Formulaire pour les Grilles (perte de charge)
 */
export class FormulaireGrille extends FormulaireBase {

    /** id of select configuring grid profile */
    private _gridProfileSelectId: string;

    /** id of select configuring grid type */
    private _gridTypeSelectId: string;

    constructor() {
        super();
        // custom variables order for results tables
        (this._formResult as FormResultFixedVar).fixedResults.variablesOrder = [
            "QMax", "CRad", "CEau", "H", "CSomGrille", "HG", "B", "S", "SPDG", "VA", "VAPDG",
            "Beta", "Alpha", "LG", "D", "DG", "SG", "VN",
            "b", "p", "e", "RFB", "REEB", "OB", "O", "OEntH", "cIncl",
            "DH00", "DH05", "DH10", "DH15", "DH20", "DH25", "DH30", "DH35", "DH40", "DH45", "DH50", "DH55", "DH60"
        ];
    }

    protected parseOptions(json: {}) {
        super.parseOptions(json);
        this._gridProfileSelectId = this.getOption(json, "gridProfileSelectId");
        this._gridTypeSelectId = this.getOption(json, "gridTypeSelectId");
    }

    public afterParseFieldset(fs: FieldSet) {
        if (this._gridTypeSelectId) {
            const sel = fs.getFormulaireNodeById(this._gridTypeSelectId);
            if (sel) {
                fs.properties.addObserver(this);
            }
        }
        if (this._gridProfileSelectId) {
            const sel = fs.getFormulaireNodeById(this._gridProfileSelectId);
            if (sel) {
                fs.properties.addObserver(this);
            }
        }
    }

    // interface Observer

    public update(sender: IObservable, data: any) {
        super.update(sender, data);
        if (data.action === "propertyChange") {
            if (data.name === "gridType") {
                this.reset();
                // Inclined grids have more input fields (OEntH and cIncl)
                this.getFieldsetById("fs_grille").updateFields();
                // Alpha and Beta are not always shown
                this.getFieldsetById("fs_plan").updateFields();
            }
            if (data.name === "gridProfile") {
                this.reset();
            }
        }
    }
}
