import { acSection, Result, CourbeRemousParams, CourbeRemous } from "jalhyd";

import { RemousResults } from "../../results/remous-results";
import { FormulaireDefinition } from "./form-definition";
import { FormCompute } from "./form-compute";
import { FormResultRemous } from "./form-result-remous";

export class FormComputeCourbeRemous extends FormCompute {

    private resultYn: Result;

    private resultYc: Result;

    constructor(formBase: FormulaireDefinition, formResult: FormResultRemous) {
        super(formBase, formResult);
    }

    private get remousResults(): RemousResults {
        const f = this._formResult as FormResultRemous;
        return f.remousResults;
    }

    protected compute() {
        this.reaffectResultComponents();
    }

    protected reaffectResultComponents() {
        const cr: CourbeRemous = this._formBase.currentNub as CourbeRemous;
        const prmCR: CourbeRemousParams = cr.prms as CourbeRemousParams;

        this.remousResults.parameters = prmCR;

        // variable supplémentaire à calculer
        this.remousResults.extraParamSymbol = this._formBase.currentNub.properties.getPropValue("varCalc");

        // calcul
        this.remousResults.result = cr.CalcSerie();

        const sect: acSection = cr.Sn;
        this.resultYn = sect.CalcSection("Yn"); // hauteur normale
        this.resultYc = sect.CalcSection("Yc"); // hauteur critique

        // données du graphe
        this.remousResults.hauteurNormale = this.resultYn.resultElement;
        this.remousResults.hauteurCritique = this.resultYc.resultElement;
        if (this.remousResults.extraParamSymbol) {
            this.remousResults.extraChart = ! ["Hs", "Hsc", "Ycor", "Ycon"].includes(this.remousResults.extraParamSymbol);
        } else {
            this.remousResults.extraChart = false;
        }
    }
}
