import { Result, MacrorugoCompound } from "jalhyd";

import { FormulaireDefinition } from "./form-definition";
import { FormCompute } from "./form-compute";
import { FormResultMacrorugoCompound } from "./form-result-macrorugo-compound";
import { NgParameter } from "../ngparam";

export class FormComputeMacrorugoCompound extends FormCompute {

    constructor(formBase: FormulaireDefinition, formResult: FormResultMacrorugoCompound) {
        super(formBase, formResult);
    }

    protected get formResult(): FormResultMacrorugoCompound {
        return this._formResult as FormResultMacrorugoCompound;
    }

    protected compute() {
        this.runNubCalc(this._formBase.currentNub);
        // reset variable index to avoid trying to access an index > 0 when nothing varies
        const mrcr = this.formResult.mrcResults;
        mrcr.variableIndex = 0;

        this.reaffectResultComponents();
    }

    protected reaffectResultComponents() {
        const mrc: MacrorugoCompound = (this._formBase.currentNub as MacrorugoCompound);
        const computedParam: NgParameter = this.getComputedParameter();
        const varParams: NgParameter[] = this.getVariatedParameters();

        // résultat de calcul de la passe à macrorugo complexe
        const mrcr = this.formResult.mrcResults;
        mrcr.calculatedParameter = computedParam;
        mrcr.result = mrc.result;
        if (varParams) {
            mrcr.variatedParameters = varParams;
        }
        // résultat de chaque enfant
        const cr: Result[] = [];
        for (const c of mrc.children) {
            cr.push(c.result);
        }
        mrcr.childrenResults = cr;
    }
}
